/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef CSERIAL_H
#define CSERIAL_H

#include <stdio.h>
#include <iostream>
#include <string.h>
#include <vector>

#if defined(__WIN32__) || defined(__CYGWIN__)		/* WIN32 Implementation */
#include <windows.h>
#else												/* Unix-like Implementation */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */
#include <sys/stat.h>
#include <sys/types.h>
#endif

#include "IStreamDaq.h"

/**
 *\file CSerial.h
 *\date  Created on: 27-mar-2009
 *\author  Author: Claudio Salvadori
 *\brief  Implementation of IStream interface to read from serial (RS232). 
*	This class reads a file and creates packet shaped 
*         like the SDef structure directives
 */

 /**
 *\class CSerial
 *\brief  Implementation of IStream interface to read from serial (RS232). 
*	This class reads a file and creates packet shaped 
*         like the SDef structure directives
 */
class CSerial: implements IStreamDaq{
	public:
		/**\brief CSerial destructor*/		
		~CSerial();

		/**\brief Get the stream definition structure from xml parser and open file
		*\param def 	Filled stream definition structure (input) 
		*\param sty		Style of reading (by length, or by start and end character)
		*\return 	s_ret value about opening the port and about the 
		*/
		virtual s_ret getDefinition (struct SDef def, s_read_style sty);
		
		/**\brief Blocking method to wait the data to be passed to formatter
		 *	  (maybe after a decode and a "divide in message" processes 
		*	  respectively to interpret the escape charactera and to divide
		*	  packets to the correct length).
		*\param buffer 	Pointer to a pointer of a vector of byte to be formatted (output)
		*\param length 	Pointer to the packet lenght variable (output)
		*\return This method return one of the possible value from s_ret \see s_ret*/
		virtual s_ret retStream(unsigned char **buffer, unsigned long *length);
		
	private:
		
		/**\brief Initialization of serial port
		*\return 	s_ret value about opening the serial port \see s_ret
		*/
		s_ret Serial_init(void);	

		/**\brief Closing of serial port*/
		void Serial_close(void);							

		/**\brief Receive from serial port
		*\param data 		Buffer to receive data
		*\param data_size 	Size of data to receive
		*\return 	s_ret value about receiving from \see s_ret
		*/
		s_ret Serial_read(unsigned char* data, unsigned int data_size);	
		
		/**\brief Send by the serial port
		*\param data 		Data buffer to send
		*\param data_size 	Size of data to send
		*\return 	s_ret value about sending from \see s_ret
		*/
		s_ret Serial_write(unsigned char* data, unsigned int data_size);	
		
		/**\brief This function execute the handshaking to initialize the stream
		*\param  style 	Handshacking style
		*\return 	s_ret value about sending from \see s_ret
		*/
		s_ret Serial_handshaking(handshake_style style);

		#if defined(__WIN32__) || defined(__CYGWIN__)		/* WIN32 Implementation */

		HANDLE 							handlerCom;	/**< Handle to serial port */
		DCB 							ctrl;		/**< Serial setting*/
		BOOL 							bPortReady;	/**< True if the port is ready*/
		COMMTIMEOUTS 						CommTimeouts;	/**< Handle of serial port */

		#else							/* Unix-like Implementation */

		int 							fd;			/**< Handle to serial port */
		struct termios                  options;	/**< Posix structure for serial communication*/
		#endif
	
		char*							se_Name;	/**< Serial name */
		unsigned long						se_Size;	/**< Packet total size */
		unsigned long						se_Size_r;	/**< Received packet total size */
		unsigned int						se_Chan;	/**< Logical channel */
		s_read_style						se_Style;	/**< Reading style*/
		handshake_style						se_HStyle;	/**< Handsaking style*/
		unsigned char						se_Start;	/**< Start character */
		unsigned char						se_End;		/**< End character */
		
		unsigned int						se_Baud;	/**< Baud rate*/
		unsigned int						se_DBits;	/**< Number of data bits*/
		std::string						se_Parity; 	/**< Parity bit*/
		float							se_StopB;	/**< Number of stop bits*/
		
		std::vector<struct SHandshaking>*			se_Handshk;	/**< Handshaking packets to start communication*/	
		

};
#endif
